@@ -1,5 +1,7 @@
 package API::Google;
-$API::Google::VERSION = '0.10';
+$API::Google::VERSION = '0.12';
+use Data::Dumper;
+
 # ABSTRACT: Perl library for easy access to Google services via their API
 
 
@@ -19,6 +21,8 @@ sub new {
   	die 'no json file specified!';
   }
   $h->{ua} = Mojo::UserAgent->new();
+  $h->{debug} = $params->{debug};
+  $h->{max_refresh_attempts} = $params->{max_refresh_attempts} || 5;
   return bless $h, $class;
 }
 
@@ -26,7 +30,7 @@ sub new {
 
 sub refresh_access_token {
   my ($self, $params) = @_;
-  warn Dumper $params;
+  warn "Attempt to refresh access_token with params: ".Dumper $params if $self->{debug};
   $params->{grant_type} = 'refresh_token';
   $self->{ua}->post('https://www.googleapis.com/oauth2/v4/token' => form => $params)->res->json; # tokens
 };
@@ -54,9 +58,10 @@ sub refresh_access_token_silent {
 		client_secret => $self->client_secret,
 		refresh_token => $self->get_refresh_token_from_storage($user)
 	});
+  warn "New tokens got" if $self->{debug};
 	my $res = {};
 	$res->{old} = $self->get_access_token_from_storage($user);
-	warn Dumper $tokens;
+	warn Dumper $tokens if $self->{debug};
 	if ($tokens->{access_token}) {
 		$self->set_access_token_to_storage($user, $tokens->{access_token});
 	}
@@ -67,7 +72,8 @@ sub refresh_access_token_silent {
 
 sub get_refresh_token_from_storage {
   my ($self, $user) = @_;
-  $self->{tokensfile}->get('gapi/tokens/'.$user.'/refresh_token');
+  warn "get_refresh_token_from_storage(".$user.")" if $self->{debug};
+  return $self->{tokensfile}->get('gapi/tokens/'.$user.'/refresh_token');
 };
 
 sub get_access_token_from_storage {
@@ -82,20 +88,76 @@ sub set_access_token_to_storage {
 
 
 
+sub build_headers {
+  my ($self, $user) = @_;
+  my $t = $self->get_access_token_from_storage($user);
+  my $headers = {};
+  $headers->{'Authorization'} = 'Bearer '.$t;
+  return $headers;
+}
+
+
+sub build_http_transaction  {
+  my ($self, $params) = @_;
+
+  warn "build_http_transaction() params : ".Dumper $params if $self->{debug};
+
+  my $headers = $self->build_headers($params->{user});
+  my $http_method = $params->{method};
+  my $tx;
+
+  if ($http_method eq 'get' || $http_method eq 'delete') {
+    $tx = $self->{ua}->build_tx(uc $http_method => $params->{route} => $headers);
+  } elsif (($http_method eq 'post') && $params->{payload}) {
+    $tx = $self->{ua}->build_tx(uc $http_method => $params->{route} => $headers => json => $params->{payload})
+  } else {
+    die 'wrong http_method on no payload if using POST';
+  }
+  return $tx;
+
+}
+
+
+
 
 sub api_query {
   my ($self, $params, $payload) = @_;
-  my %headers = (
-    'Authorization' => 'Bearer '.$self->get_access_token_from_storage($params->{user})
-  );
-  my $http_method = $params->{method};
-  if ($http_method eq 'get' || $http_method eq 'delete') {
-    return $self->{ua}->$http_method($params->{route} => \%headers)->res->json;
-  } elsif (($http_method eq 'post') && $payload) {
-    return $self->{ua}->$http_method($params->{route} => \%headers => json => $payload)->res->json;
-  } else {
-    die 'wrong http_method';
+
+  warn "api_query() params : ".Dumper $params if $self->{debug};
+
+  $payload = { payload => $payload };
+  %$params = (%$params, %$payload);
+
+  my $tx = $self->build_http_transaction($params);
+  my $res = $self->{ua}->start($tx)->res->json;
+    
+  # for future:
+  # if ( grep { $_->{message} eq 'Invalid Credentials' && $_->{reason} eq 'authError'} @{$res->{error}{errors}} ) { ... }
+
+  warn "First api_query() result : ".Dumper $res if $self->{debug};
+
+  if (defined $res->{error}) { # token expired error handling
+
+    my $attempt = 1;
+
+    while ($res->{error}{message} eq 'Invalid Credentials')  {
+      if ($attempt == $self->{max_refresh_attempts}) {
+        last;
+      }
+      warn "Seems like access_token was expired. Attemptimg update it automatically ..." if $self->{debug};
+      $self->refresh_access_token_silent($params->{user});
+      $tx = $self->build_http_transaction($params);
+      $res = $self->{ua}->start($tx)->res->json;
+      $attempt++;
+    }
+
+    if ($attempt > 1) {
+      warn "access_token was automatically refreshed";
+    }
+
   }
+
+  return $res;
 };
 
 
@@ -113,7 +175,7 @@ API::Google - Perl library for easy access to Google services via their API
 
 =head1 VERSION
 
-version 0.10
+version 0.12
 
 =head1 SYNOPSIS
 
@@ -155,6 +217,21 @@ config.json must be structured like:
 
 Get new access token for user from Google API server and store it in jsonfile
 
+=head2 build_headers
+
+Keep access_token in headers always actual 
+
+$gapi->build_http_transactio($user);
+
+=head2 build_http_transaction 
+
+$gapi->build_http_transaction({ 
+  user => 'someuser@gmail.com',
+  method => 'post',
+  route => 'https://www.googleapis.com/calendar/users/me/calendarList',
+  payload => { key => value }
+})
+
 =head2 api_query
 
 Low-level method that can make API query to any Google service
