@@ -1,5 +1,5 @@
 package API::Google::GCal;
-$API::Google::GCal::VERSION = '0.10';
+$API::Google::GCal::VERSION = '0.12';
 
 use parent 'API::Google';
 # use base 'API::Google';
@@ -78,11 +78,18 @@ sub busy_time_ranges {
 
 sub events_list {
    my ($self, $params) = @_;
-    $self->api_query({ 
+
+   if (!defined $params->{calendarId}) { die "No calendarId provided as parameter"}
+   if (!defined $params->{user}) { die "No user  provided as parameter"}
+   
+   my $res = $self->api_query({ 
       method => 'get', 
       route => $self->{api_base}.'/calendars/'.$params->{calendarId}.'/events',
       user => $params->{user}
-    })->{items};
+    });
+
+   if (defined $res->{items}) { return $res->{items} };
+   if (defined $res->{error}) { return $res };
 };
 
 
@@ -102,7 +109,7 @@ API::Google::GCal - Google Calendar API client
 
 =head1 VERSION
 
-version 0.10
+version 0.12
 
 =head1 SYNOPSIS
 
@@ -150,10 +157,14 @@ Get all calendars of particular Google account
 
 =head2 get_calendar_id_by_name
 
+$gapi->get_calendar_id_by_name($user, $name)
+
 Get calendar id by its name. Name = "summary" parameter
 
 =head2 add_event
 
+$gapi->add_event($user, $calendar_id, $event_data)
+
 # https://developers.google.com/google-apps/calendar/v3/reference/events/insert
 
 =head2 busy_time_ranges
@@ -169,7 +180,7 @@ L<https://developers.google.com/google-apps/calendar/v3/reference/events/list>
 Usage:
 
 $gapi->events_list({
-  calendarID => 'ooqfhagr1a91u1510ffdf7vfpk@group.calendar.google.com',
+  calendarId => 'ooqfhagr1a91u1510ffdf7vfpk@group.calendar.google.com',
   user => 'someuser@gmail.com'
 });
 
